;AUDIO.ASM
;
;*******************************************************************************
;*REVISION HISTORY:
;*
;*DATE             	REV. NO.      DESCRIPTION
;*
;*September 16, 2017	1.00          Adapt9S12P128 Demo
;*
;*
;*Author: Exequiel Rarama for the Adapt9S12P Demo
;*******************************************************************************
;
;Compiled CW5.1/5.2
;
;
; ---------------------------
; Demo - Audio/Beeper Routine
; ---------------------------

	include derivative.inc
;
;

;Public Function
	XDEF check_audio
	XDEF audio_init
	XDEF set_audio
	XDEF AudioOn
	XDEF audio_int

	XDEF PlayAudioInit
	XDEF BeepingDisabled

;Public Variables
	XDEF audio_timer	;audio timer for the beeping sound
	XDEF audio_period
	XDEF audio_status

	XDEF audio_delay
	XDEF audio_counter
	XDEF audio_state

;Public Tones
	XDEF pwr_on_tone

;External Function
	XREF OutStr0


;External variables

; variable/data section
MY_EXTENDED_RAM: SECTION
; Insert here your data definition.
* System Variables

;Audio Variables
audio_timer:	ds	1	            	;audio tone duration timer
audio_ptr:	ds	2	            	;pointer to audio tone table
audio_period:	ds	2	            	;audio tone period
;
rep_addr:	ds	2	            	;repetition address
rep_count:	ds	1	            	;repetition counter
audio_status:	ds	1

audio_delay:	ds	2
audio_counter:	ds	1
audio_state:	ds	2

AudioTblPtr:	ds	2
AudioFlg:	ds	1

;
OC4mask1:	equ	%00010000       ;IOS4 = 1, Bit 4 as output compare
OC4mask2:	equ	%00010000       ;C4I = 1, Enable Interrupt
OC4mask3:	equ	%00000001       ;OM4 = 0, OL4 = 1, toggel OC output line
OC4flag:	equ	%00010000       ;C4F = 1 to clear Interrupt flag

OM4:		equ	%10
OL4:		equ	%01

AudioControl:	equ	%00000100	;Port J bit 2 to control Audio
Seconds:	equ	3906		;Of the RTI rate

;***********************************************************************************************
; code section
NON_BANKED:SECTION
;
;
; Initialization
; --------------
;
audio_init:
		movw	#$1ff,audio_period
		movb	#$ff,audio_timer
		movb	#2,audio_counter	    	;beep 2 times
		movw	#00,audio_delay
		clr	audio_status

		bset	DDRJ,AudioControl	    	;Bit 0 output
		bclr	PTJ,AudioControl	    	;Bit 0 = 0
            
		bset	TSCR1,%10000000		    	;TEN=1 - Enable timer
		bset	TIOS,OC4mask1
		bset	TIE,OC4mask2		      	;enable OC4 interrupt

		bset	TFLG1, #OC4flag		      	;clear OC4 flag
		movw	#AudioOn,audio_state
		clr	AudioFlg

audio_ready:
		rts


AudioOn:
		ldx	audio_delay
		bne	AudioEx

		ldx	#pwr_on_tone
		jsr	set_audio                 	;Make Power on Sound

		movw	#Seconds/2,audio_delay    	;500ms pause
		movw	#audio_ready,audio_state

AudioEx:
		rts

;--------------------------------------------------------------------
PlayAudioInit:
		ldx	#AudioTable0
		stx	AudioTblPtr

		movw	#PlayAudio,audio_state
		movw	#00,audio_delay		
  
		ldx	#AudioChaserMsg		        ;Send Menu message
		jsr	OutStr0

		clr	AudioFlg
		rts

AudioChaserMsg:	dc.b	$A,$D,'Audio/Beeper enabled',0


PlayAudio:
		ldx	audio_delay
		bne	PlayAudioEx

		ldaa	AudioFlg
		beq	Audio1

		cmpa	#1
		beq	Audio2	

		cmpa	#2
		beq	Audio3

		cmpa	#3
		beq	Audio4

		clr	AudioFlg

Audio1:
		ldx	#AudioTable0
		jsr	set_audio		        ;Make Power on Sound
		inc	AudioFlg

		bra	PlayAudio10

Audio2:
		ldx	#AudioTable1
		jsr	set_audio	            	;Make Power on Sound
		inc	AudioFlg
		bra	PlayAudio10

Audio3:
		ldx	#AudioTable2
		jsr	set_audio	            	;Make Power on Sound
		inc	AudioFlg
		bra	PlayAudio10

Audio4:
		ldx	#AudioTable3
		jsr	set_audio		        ;Make Power on Sound
		inc	AudioFlg

PlayAudio10:
		movw	#Seconds/2,audio_delay		;500ms pause

PlayAudioEx:
		rts


;--------------------------------------------------------------------
BeepingDisabled:
		movw	#00,audio_delay
		jsr	AudioOn

		ldx	#AudioDisabledMsg	    	;Send Menu message
		jsr	OutStr0

		clr	AudioFlg
		rts

AudioDisabledMsg:	dc.b	$A,$D,'Audio/Beeper disabled',0

;
;-----------------------------------------------------------------------------------------------
;
set_audio:
		sei
		stx	audio_ptr
		clr	audio_timer
		cli
		rts
;
;------------
; Check Audio
;------------
check_audio:
		ldaa	audio_timer             ;decrement audio duration timer
		beq	aud05                   ;  unless already timed out (0)

		inca                              ;  OR infinite duration (FF)
		beq	audex

		dec	audio_timer
		bne	audex

aud05:
		ldx	audio_ptr               ;get pointer to audio table
		ldaa	0,x                     ;get next tone duration time & skip
		staa	audio_timer             ;  if not a repetition indicator
		bne	aud20                   ;  (0 = repetition indicator)

		ldaa	1,x                     ;skip if end of repetition (2nd byte
		beq	aud10                   ;  = 0)
		staa	rep_count               ;else, start of repetition:  store
		inx                               ;  number of repetitions & starting
		inx                               ;  address
		stx	rep_addr
		stx	audio_ptr
		bra	audex
;
aud10:
		ldaa	rep_count
		inca
		beq	aud15
		dec	rep_count               ;end of repetition: skip if repetition
		beq	aud50                   ;  counter has counted down
;
aud15:
		ldx	rep_addr                ;else, restore 'start of repetition'
		stx	audio_ptr               ;  address
		bra	audex
;
aud20:
		ldd	1,x                     ;set new audio tone period & duration
		std	audio_period            ;  audio is off (=0)
		beq	aud30

		sei				                        ;disable interrupt a bit
		addd	TC4
		std	TC4
		cli				                        ;reenable interrupt again
		bra	aud40

aud30:
		bclr	PTJ,AudioControl

aud40:
		inx                               ;increment and save audio table

aud50:
		inx                               ;  pointer
		inx
		stx	audio_ptr

audex:
		rts
;

;----------------------------------------------------------------------------
; Audio Interrupt Routine
;------------------------
audio_int:
		ldd	audio_period	          ;audio time period for next
		addd	TC4		                	;  output compare interrupt
		std	TC4			

		ldx	audio_period
		beq	auiex

		ldaa	audio_status
		eora	#1
		staa	audio_status
		beq	aui10

		bclr	PTJ,AudioControl
		bra	auiex

aui10:
		bset	PTJ,AudioControl

auiex:
		bset	TFLG1, #OC4flag		      ;clear OC4 flag
		rti

;============================================================
AudioTable0:
	          dc.b	40,40,40
	          dc.b	40,40,40
	          dc.b	40,40,40
	          dc.b	40,40,40

	          dc.b	40,40,40
	          dc.b	40,40,40
	          dc.b	40,40,40
	          dc.b	40,40,40

	          dc.b	$ff,0,0

AudioTable1:
	          dc.b	60,60,60
	          dc.b	60,60,60
	          dc.b	60,60,60
	          dc.b	60,60,60

	          dc.b	60,60,60
	          dc.b	60,60,60
	          dc.b	60,60,60
	          dc.b	60,60,60

	          dc.b	$ff,0,0

AudioTable2:
	          dc.b	80,80,80
	          dc.b	80,80,80
	          dc.b	80,80,80
	          dc.b	80,80,80

	          dc.b	80,80,80
	          dc.b	80,80,80
	          dc.b	80,80,80
	          dc.b	80,80,80

	          dc.b	$ff,0,0

AudioTable3:
	          dc.b	100,100,100
	          dc.b	100,100,100
	          dc.b	100,100,100
	          dc.b	100,100,100

	          dc.b	100,100,100
	          dc.b	100,100,100
	          dc.b	100,100,100
	          dc.b	100,100,100

	          dc.b	$ff,0,0

;----------------------------------------------------------------------------
pwr_on_tone:
	          dc.b	40,40,40
	          dc.b	40,80,255
	          dc.b	40,40,40

	          dc.b	220,80,0
	          dc.b	240,60,255

	          dc.b	160,100,0
	          dc.b	160,100,0
	          dc.b	160,80,200

	          dc.b	220,80,0
	          dc.b	240,60,255

	          dc.b	$ff,0,0

	          dc.b	20,3,20		            	;middle number (3) denotes 
	          dc.b	20,3,$50	            	; frequency from Output compare
	          dc.b	20,3,20

	          dc.b	$ff,0,0


	          END
