;INT.ASM
;
;*******************************************************************************
;*REVISION HISTORY:
;*
;*DATE             	REV. NO.      DESCRIPTION
;*
;*September 16, 2017	1.00          Adapt9S12P128 Demo
;*
;*
;*Author: Exequiel Rarama for the Adapt9S12P Demo
;*******************************************************************************
;
;Compiled CW5.1/5.2
;
; ---------------------------
; Demoboard - ISR Routine
; ---------------------------

	include derivative.inc


;Public Function
	XDEF RealTimeInit
	XDEF RealTimeInt

	XDEF OutStr0
	XDEF SerOutput0
	XDEF SerInputInt0
  
;Public Variables
	XDEF wait_timer

	XDEF ad0
	XDEF ad1
	XDEF ad2
	XDEF ad3
	
;External Function

	XREF check_audio

;External Variables
	XREF state_timer

	XREF Command
	XREF CommandFlg
	XREF audio_delay

	XREF WiFITimer	


; variable/data section
MY_EXTENDED_RAM: SECTION
; Insert here your data definition.
* System Variables

;
wait_timer:     ds      2

ad0:            ds      2
ad1:            ds      2
ad2:            ds      2
ad3:            ds      2

;------------------------------------------------------------------------------
;ATD Variables
admask2:        equ     %11000000       ;AFFC,ADPU=1 - Enable Analog to Digital
admask3         equ     %00000000       ;FRZ1,FRZ0=0
admask4         equ     %10000001       ;SMP1,SMP0 = 0; S10BM,PRS0=1 - Select Sample time adn Bit mode
admask5         equ     %01110000       ;S8CM = 1, SCAN = 1, MULT = 1
SCFflag         equ     %10000000       ;SCF - Sequence Complete flag

;RTI Variables
clrmask         equ     %11000000       ;mask for clearing timer flags

ms0256        	equ   	%00010000	      ;RTI = 8MHz/(2^10) = 0.256ms
ms0512	      	equ   	%00100000	      ;RTI = 8MHz/(2^11) = 0.512ms
ms1024	      	equ   	%00110000	      ;RTI = 8MHz/(2^12) = 1.024ms
ms2048	      	equ	%01000000     	;RTI = 8MHz/(2^13) = 2.048ms
ms4096	      	equ   	%01010000	      ;RTI = 8MHz/(2^14) = 4.096ms

ms1000:	      	equ   	%00000000       ;1.000ms

RTIF:		equ     %10000000
RTIE:		equ     %10000000
RTIOSCSEL:      equ     %00000010
RTDEC:          equ     %10000000       ; Decimal based divider value. See Table 10-11

;SCI Variables
scimask         equ     %00101100       ;RIE - SCI Interrupt enable
                                        ;RE - Receiver Enable
RDRFflag        equ     %00100000       ;RDRF - Receive Data Register Full flag
TDREflag        equ     %10000000       ;TDRE - Transmit Data Register Empty flag

;Baud rate definitions
OscFreq:	equ	8000000		    ;Enter Osc speed
SYNDIV:	        equ	$02		      ; mult by synr + 1 = 3 (24MHz)
REFDIV:         equ	$00		      ;
BusFreq:        equ	((OscFreq/(REFDIV+1))*(SYNDIV+1))

;SCI baud rate = SCI bus clock / (16 * SCIBR[12:0])

baud230400:     equ     BusFreq/(16*230400)  			;sets baud rate to 230,400
baud115200:     equ     BusFreq/(16*115200)  			;sets baud rate to 115,200
baud14400:      equ     BusFreq/(16*14400)   			;sets baud rate to 014,400
baud9600:       equ     BusFreq/(16*9600)  			  ;sets baud rate to 009,600


* Operational Constants

TRUE:           equ     $FF
FALSE:          equ     $00
CR:             equ     $0D
LF:             equ     $0A
SPACE:          equ     $20

TCIE:           equ	$40
ILIE:           equ	$10
RIE:            equ   	%00100000                 ;Receiver Full Interrupt Enable Bit, RIE enables the receive data register full flag, RDRF, or the overrun flag,
                                                  ;OR, to generate interrupt requests
TE:             equ     %00001000                 ;Transmitter Enable Bit, TE enables the SCI transmitter and configures the TXD pin as being controlled by
                                                  ;the SCI. The TE bit can be used to queue an idle preamble.
RE:		equ   	%00000100                 ;Receiver Enable Bit, RE enables the SCI receiver
RWU:		equ   	$02
SBK:		equ   	$01

;------------------------------------------------------------------------------

***************************** Program *******************************

; code section
NON_BANKED:SECTION
;
RealTimeInit:						;Initialize Real Time Interrupt
		bset	CPMUCLKS, #RTIOSCSEL		;RTI clock source is OSCCLK
		movb	#RTDEC+ms1000,CPMURTI		;Decimal based divider value. See Table 10-11 
							;and initialize RTI rate
		bset	CPMUFLG, #RTIF			;clear RTI flag
		bset	CPMUINT, #RTIE			;Enable RTI


;Initialize Analog To Digital
		movw	#$03F0,ATDDIEN			;Make Bit 4 to 9 as buffered Digital input

;		movb	#%00000000, ATDCTL0		;Wrap Around Channel Select Bits - Wrap to and begin at AN0
		movb	#%01000000, ATDCTL1		;SRES1 = 1 set for 12-bit resolution bit 
		movb	#%01000000, ATDCTL2		;ATD Fast Flag Clear All bit 6 = 1, ETRIGE bit 2 = 0
		movb	#%10100000, ATDCTL3		;right justified, 4 sample per sequence
		movb	#%00000010, ATDCTL4		;prescaler = 2; ATD clock = 6.25MHz / (2 * (2 + 1)) == 1.04MHz
		movb	#%00110000, ATDCTL5		;SCAN bit 5 and MULT bit 4 = 1, 

;Initialize first Serial Communication Interface 0
;
		movb	#0,SCIBDH
		movb	#baud115200,SCIBDL		;..BDH=0 so baud = 115200
		movb	#0,SCICR1
		movb	#TE+RE+RIE,SCICR2		;RIE, RE and TE on
		rts

;------------------------------------------------------------------------------
* Real-time Interrupt Routine
;
RealTimeInt:          
		brclr	ATDSTAT0,SCFflag,*		;Loop here until SCF of ATD is set
							;save ATD

		ldd	ATDDR0H
		std	ad0

		ldd	ad0
		lsrd	
		lsrd
		stab	PWMDTY4
		stab	PWMDTY5

		ldd	ATDDR1H
		std	ad1

		ldd	ATDDR2H
		std	ad2

		ldd	ATDDR3H
		std	ad3

RTI0:
		ldx	state_timer
		beq	RTI1
		dex
		stx	state_timer

RTI1:
		ldx	wait_timer
		beq	RTI2

		dex
		stx	wait_timer

RTI2:

RTI3:
RTI4:
RTI5:
		ldx	audio_delay
		beq	RTI6

		dex
		stx	audio_delay

RTI6:
		jsr	check_audio

		bset	CPMUFLG, #RTIF		            ;clear RTI flag
		rti


;==============================================================================
;SCI0 Send Character strings
;
OutStr0:            				          ; send a null terminated string to the display.
		ldaa	1,x+              ; get a character, advance pointer, null?
		beq	OutStrDone        ; yes. return.
		bsr	SerOutput0        ; no. send it out the SCI.
		bra	OutStr0           ; go get the next character.
;
OutStrDone:
		rts

;------------------------------------------------------------------------------
SerOutput0:
		brclr	SCISR1,TDREflag,SerOutput0     ;check if buffer is empty
		staa	SCIDRL
		rts

;  SCI0 Input Interrupt Handler
;  Gets bytes from SCI0.
;  Sets COMMAND_PENDING flag.

SerInputInt0:
		ldaa	SCISR1			        ;read register to clear flag RDRF
		movb	SCIDRL,Command		  ;read receive buffer
		movb	#1,CommandFlg		    ;Set for new data

		rti


	          END

