;Main.asm
;
;*******************************************************************************
;*REVISION HISTORY:
;*
;*DATE             	REV. NO.      DESCRIPTION
;*
;*September 16, 2017	1.00          Adapt9S12P128 Demo
;*
;*
;*Author: Exequiel Rarama for the Adapt9S12P Demo
;*******************************************************************************
;
;Compiled CW5.1/5.2
;
; Include derivative-specific definitions
            INCLUDE 'derivative.inc'

; export symbols
            XDEF Entry, _Startup, ResetFunc
            ; we use export 'Entry' as symbol. This allows us to
            ; reference 'Entry' either in the linker .prm file
            ; or from C/C++ later on


;Public Function
	XDEF ResetFunc
	XDEF Entry
	XDEF delay
	XDEF small_delay

;Public Variables
	XDEF Command
	XDEF CommandFlg

;External Function

	XREF RealTimeInit			;Initialize RTI
	XREF RealTimeInt

	XREF goPower
	XREF ShowMenu
	XREF ProcessCommand
	XREF audio_init

	XREF audio_int
	XREF SerInputInt0
  
;External Variables

	XREF state
	XREF audio_state

  XREF __SEG_END_SSTACK      ; symbol defined by the linker for the end of the stack
            
; variable/data section
MY_EXTENDED_RAM: SECTION
; Insert here your data definition.
* System Variables

Command:	ds	1		          ;used by ProcessCommand
CommandFlg:	ds	1		          ;used by ProcessCommand

*  Operational Parameters

RAM:		equ	$1400			    ;9S12GA240 internal RAM
STACK:		equ	$4000			    ;Stack at top of internal ram
EEPROM:		equ	$400			    ;EEPROM start address
FLASH:		equ	$4000			    ;Flash start address

OscFreq:	equ	8000  	      ;Enter Osc speed
SYNDIV:		equ	$02		        ; mult by synr + 1 = 3 (24MHz)
REFDIV:		equ	$00		        ;

NECLK:		equ	%00010000     ;Port E bit 4 as I/O
OSCE:		equ	%10000000     ;Bit 7 of CPMUOSC register
OSCPINS_EN:	equ	%00100000     ;Bit 5 of CPMUOSC register
REFFRQ:		equ	%10000000     ;(6MHZ<FREF<=12MHZ)
LOCK:		equ	%00001000     ;lock status bit
UPOSC:		equ	%00000001     ;Oscillator Status Bit
PLLSEL:		equ	%10000000     ;PLL select bit
PLLON:		equ	%01000000     ;phase lock loop on bit
initSYNR:	equ	$02           ; mult by synr + 1 = 3 (24MHz)
VCOFRQ:		equ	$40           ; vco gain
initREFDV:	equ	$00
initPOSTDIV:	equ	$00           ;Post Divider Register (CPMUPOSTDIV)

BusFreq:	equ	((OscFreq/(REFDIV+1))*(SYNDIV+1))


; code section
NON_BANKED:SECTION
_Startup:
Entry:
ResetFunc: 
		sei					;Disable Any interrupts
		bset	ECLKCTL, #ECLKCTL_NECLK		;Port E bit 4 as I/O - not free running E clock

;Initialize Stack
		lds	#__SEG_END_SSTACK		;initialize stack pointer
		lds	#$4000				;Stack at top of internal ram
	           
;Internal RC Oscillator (IRC)
;Trimmable internal reference clock.
;Frequency: 1 MHz
;Trimmed accuracy over 40C to +125C ambient temperature range:
;External OSC = 8.000MHz
;Initialize clock generator and PLL
;Fref = Fosc/(REFDIV+1)
;Fref = 8000000/(0+1)
;VCOFRQ[1:0] = 00
;REFFRQ[1:0] = 10 (6MHZ<FREF<=12MHZ)
;VCO 2 fREF If PLL has locked (LOCK=1) = FVCO = 2*Fref*(SYNDIV+1)

		movb	#initSYNR+VCOFRQ,CPMUSYNR	;set PLL multiplier
		movb	#initREFDV+REFFRQ,CPMUREFDIV	;set PLL divider
		movb	#initPOSTDIV,CPMUPOSTDIV	;set post-divider
		movb	#$C0,CPMUOSC			;Enable external oscillator
		nop
		nop
		brclr	CPMUFLG,LOCK,*+0		;while (!(crg.crgflg.bit.lock==1))
		bset	CPMUCLKS,PLLSEL			;engage PLL to system

		jsr	RealTimeInit			;Initialize SCI and RTI
		jsr	audio_init			;Initialize PTT bit 0 and OC4 as audio o/p

 		clr	CommandFlg
          	cli					;unmask interrupts
		jsr	goPower
          	jsr	ShowMenu
            

;------------------------------------------------------------------------------
main					                ;Main Loop
		jsr	ProcessCommand			;Check if there are new command to execute

		ldy	state
		jsr	0,y

		ldy	audio_state			;Process audio command
		jsr	0,y

          	bra	main

;------------------------------------------------------------------------------
delay:
		pshy
		ldy	#0
		bra	dly

small_delay:
		pshy
		ldy	#777

dly:
		dbne	y,dly
		puly
		rts


;------------------------------------------------------------------------------
;Vector address
;

            ORG $FF80
            
VECTOR63       dc.w ResetFunc	;//	;Reserve
VECTOR62       dc.w ResetFunc	;//	;Reserve
VECTOR61       dc.w ResetFunc	;//	;Reserve
VECTOR60       dc.w ResetFunc	;//	;Reserve

VECTOR59       dc.w ResetFunc	;//	;PWM Emergency Shutdown
VECTOR58       dc.w ResetFunc	;//	;VREG LVI
VECTOR57       dc.w ResetFunc	;//	;Port P
VECTOR56       dc.w ResetFunc	;//	;Reserved
VECTOR55       dc.w ResetFunc	;//	;Reserved
VECTOR54       dc.w ResetFunc	;// 	;Reserved
VECTOR53       dc.w ResetFunc	;//	;Reserved
VECTOR52       dc.w ResetFunc	;//	;Reserved
VECTOR51       dc.w ResetFunc	;//	;Reserved
VECTOR50       dc.w ResetFunc	;//	;Reserved
VECTOR49       dc.w ResetFunc	;//	;Reserved
VECTOR48       dc.w ResetFunc	;//	;Reserved
VECTOR47       dc.w ResetFunc	;//	;Reserved
VECTOR46       dc.w ResetFunc	;//	;Reserved
VECTOR45       dc.w ResetFunc	;//	;Reserved
VECTOR44       dc.w ResetFunc	;//	;Reserved
VECTOR43       dc.w ResetFunc	;//	;Reserved

VECTOR42       dc.w ResetFunc	;//	;Reserved
VECTOR41       dc.w ResetFunc	;//	;Reserved
VECTOR40       dc.w ResetFunc	;//	;Reserved
VECTOR39       dc.w ResetFunc	;//	;CAN transmit
VECTOR38       dc.w ResetFunc	;//	;CAN receive
VECTOR37       dc.w ResetFunc	;//	;CAN errors
VECTOR36       dc.w ResetFunc	;//	;CAN wake-up
VECTOR35       dc.w ResetFunc	;//	;FLASH

VECTOR34       dc.w ResetFunc	;//	;Reserved
VECTOR33       dc.w ResetFunc	;//	;Reserve
VECTOR32       dc.w ResetFunc	;//	;Reserve

VECTOR31       dc.w ResetFunc	;//	;Reserved
VECTOR30       dc.w ResetFunc	;//	;SCI2

VECTOR29       dc.w ResetFunc	;//	;CRG Self Clock Mode
VECTOR28       dc.w ResetFunc	;//	;CRG PLL lock
VECTOR27       dc.w ResetFunc	;//	;Reserved
VECTOR26       dc.w ResetFunc	;//	;Reserved
VECTOR25       dc.w ResetFunc	;//	;Reserved

VECTOR24       dc.w ResetFunc	;//	;Port J (PIEP)
VECTOR23       dc.w ResetFunc	;//	;Reserved
VECTOR22       dc.w ResetFunc	;//	;ATD (ATDCTL2 - ASCIE)
VECTOR21       dc.w ResetFunc	;//	;SCI1
VECTOR20       dc.w SerInputInt0	;//		;SCI0
VECTOR19       dc.w ResetFunc	;//	;SPI
VECTOR18       dc.w ResetFunc	;//	;Pulse Accumulator 0 input edge
VECTOR17       dc.w ResetFunc	;//	;Pulse Accumulator 0 overflow
VECTOR16       dc.w ResetFunc	;//	;Standard Timer 0 Overflow
VECTOR15       dc.w ResetFunc	;//	;Timer 0 Channel 7
VECTOR14       dc.w ResetFunc	;//	;Timer 0 Channel 6
VECTOR13       dc.w ResetFunc	;//	;Timer 0 Channel 5
VECTOR12       dc.w audio_int	;//	;Timer 0 Channel 4

VECTOR11       dc.w ResetFunc	;//	;Timer 0 Channel 3
VECTOR10       dc.w ResetFunc	;//	;Timer 0 Channel 2
VECTOR09       dc.w ResetFunc	;//	;Timer 0 Channel 1
VECTOR08       dc.w ResetFunc	;//	;Timer 0 Channel 0

VECTOR07       dc.w RealTimeInt	;//	;Real Time Interrupt
VECTOR06       dc.w ResetFunc	;//	;IRQ
VECTOR05       dc.w ResetFunc	;//	;XIRQ
VECTOR04       dc.w ResetFunc	;//	;SWI
VECTOR03       dc.w ResetFunc	;//	;Instruction Trap
VECTOR02       dc.w ResetFunc	;//	;COP failure
VECTOR01       dc.w ResetFunc	;//	;Clock Monitor

	          END
